/*******************************************************************************
 * Copyright (c) 2013, Perforce Software
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package com.perforce.search.test;

import java.io.File;

import org.apache.log4j.Logger;
import org.apache.solr.client.solrj.SolrQuery;
import org.apache.solr.client.solrj.SolrServerException;
import org.apache.solr.client.solrj.response.QueryResponse;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.mock.web.MockHttpServletResponse;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import com.perforce.p4java.extension.exception.ServerAccessException;
import com.perforce.p4java.extension.exception.ServerConnectionException;
import com.perforce.p4java.extension.exception.ServerErrorException;
import com.perforce.p4java.extension.operation.CounterCommand;
import com.perforce.p4java.extension.server.Server;
import com.perforce.search.test.util.IndexTestBase;

public class NoScanTest extends IndexTestBase {
    static Logger log = Logger.getLogger(NoScanTest.class);

    @BeforeClass
    public void setUp() throws Exception {
	this.configFileLocation = "/noscan.config";
	// we scan later, but setUp() will start the scanner
	this.scanAndQuit = false;
	super.setUp();
    }

    @Test(groups = "init")
    void testEmptyQuery() {
	try {
	    SolrQuery q = new SolrQuery();
	    q.setQuery("*:*");
	    QueryResponse r;
	    r = this.solrServer.query(q);
	    // make nothing is indexed
	    Assert.assertEquals(r.getResults().getNumFound(), 0);
	} catch (SolrServerException e) {
	    e.printStackTrace();
	    Assert.fail("Unhandled server exception");
	}

    }

    @Test(groups = "addsome", dependsOnGroups = "init")
    void testAddSomeChanges() throws ServerConnectionException,
	    ServerAccessException, ServerErrorException, InterruptedException {
	// exercise the controller: send models with the change number of
	// all of the changelists
	Server server = null;
	try {
	    server = this.pool.acquire();
	    int changeNum = Integer.valueOf(CounterCommand.getCounterValue(
		    "change", server));
	    // don't really care about the req/response, make a controller
	    // tester for that
	    MockHttpServletRequest req = new MockHttpServletRequest();
	    MockHttpServletResponse resp = new MockHttpServletResponse();
	    for (int i = 1; i <= changeNum; ++i) {
		this.queueController.queueTask(req, resp,
			config.getQueueToken(),
			"change-commit," + String.valueOf(i));
	    }
	    // wait until the queue temp folder is empty
	    File queueFile = new File(config.getQueueDir());
	    while (queueFile.listFiles().length > 0) {
		log.debug("waiting for queue manager to catch up");
		Thread.sleep(1000);
	    }
	    this.scanAndQuit = true; // allow file threads to die
	    this.queueManager.waitForFileThreadsCompletion();
	    // check the counts
	    try {
		SolrQuery q = new SolrQuery();
		q.setQuery("*:*");
		QueryResponse r;
		r = this.solrServer.query(q);
		// make sure every file revision was indexed
		Assert.assertEquals(r.getResults().getNumFound(),
			EXPECTED_INDEX_REV_COUNT);
	    } catch (SolrServerException e) {
		e.printStackTrace();
		Assert.fail("Unhandled server exception");
	    }
	} finally {
	    this.pool.release(server);
	}
    }
}
